# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class RelativeToDateTime(str, Enum):
    """
    RelativeToDateTime
    """

    """
    allowed enum values
    """
    UNDEFINED = 'Undefined'
    BEFOREORON = 'BeforeOrOn'
    BEFORE = 'Before'
    ONDAYOF = 'OnDayOf'
    AFTERORON = 'AfterOrOn'
    AFTER = 'After'
    EXACTLY = 'Exactly'

    @classmethod
    def from_json(cls, json_str: str) -> RelativeToDateTime:
        """Create an instance of RelativeToDateTime from a JSON string"""
        return RelativeToDateTime(json.loads(json_str))
