# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class PointInTimeSpecification(str, Enum):
    """
    PointInTimeSpecification
    """

    """
    allowed enum values
    """
    UNDEFINED = 'Undefined'
    NOW = 'Now'
    FIRSTOFMONTH = 'FirstOfMonth'
    FIRSTBUSINESSDAYOFTHEMONTH = 'FirstBusinessDayOfTheMonth'
    LASTDAYOFTHEMONTH = 'LastDayOfTheMonth'
    LASTBUSINESSDAYOFMONTH = 'LastBusinessDayOfMonth'
    FIRSTDAYOFYEAR = 'FirstDayOfYear'
    LASTDAYOFYEAR = 'LastDayOfYear'

    @classmethod
    def from_json(cls, json_str: str) -> PointInTimeSpecification:
        """Create an instance of PointInTimeSpecification from a JSON string"""
        return PointInTimeSpecification(json.loads(json_str))
