# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic.v1 import BaseModel, StrictStr

class KeyValuePairOfStringToString(BaseModel):
    """
    KeyValuePairOfStringToString
    """
    key: Optional[StrictStr] = None
    value: Optional[StrictStr] = None
    __properties = ["key", "value"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> KeyValuePairOfStringToString:
        """Create an instance of KeyValuePairOfStringToString from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if key (nullable) is None
        # and __fields_set__ contains the field
        if self.key is None and "key" in self.__fields_set__:
            _dict['key'] = None

        # set to None if value (nullable) is None
        # and __fields_set__ contains the field
        if self.value is None and "value" in self.__fields_set__:
            _dict['value'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> KeyValuePairOfStringToString:
        """Create an instance of KeyValuePairOfStringToString from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return KeyValuePairOfStringToString.parse_obj(obj)

        _obj = KeyValuePairOfStringToString.parse_obj({
            "key": obj.get("key"),
            "value": obj.get("value")
        })
        return _obj
