# coding: utf-8

"""
    FINBOURNE Access Management API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class DateQuality(str, Enum):
    """
    DateQuality
    """

    """
    allowed enum values
    """
    UNDEFINED = 'Undefined'
    ISFIRSTDAYOFANYMONTH = 'IsFirstDayOfAnyMonth'
    ISLASTDAYOFANYMONTH = 'IsLastDayOfAnyMonth'
    ISBUSINESSDAYOFANYMONTH = 'IsBusinessDayOfAnyMonth'
    ISFIRSTDAYOFTHECURRENTMONTH = 'IsFirstDayOfTheCurrentMonth'
    ISLASTDAYOFTHECURRENTMONTH = 'IsLastDayOfTheCurrentMonth'
    ISBUSINESSDAYOFTHECURRENTMONTH = 'IsBusinessDayOfTheCurrentMonth'
    ISBEFOREMIDDAY = 'IsBeforeMidday'
    ISBEFORE5PM = 'IsBefore5pm'
    ISAFTER5PM = 'IsAfter5pm'

    @classmethod
    def from_json(cls, json_str: str) -> DateQuality:
        """Create an instance of DateQuality from a JSON string"""
        return DateQuality(json.loads(json_str))
